Package: sctransform
Type: Package
Title: Variance Stabilizing Transformations for Single Cell UMI Data
Version: 0.4.3
Date: 2025-12-25
Authors@R: c(
  person(given = "Christoph", family = "Hafemeister", email = "christoph.hafemeister@nyu.edu", role = "aut", comment = c(ORCID = "0000-0001-6365-8254")),
  person(given = "Saket", family = "Choudhary", email = "saketc@iitb.ac.in", role = c("aut", "cre"), comment = c(ORCID = "0000-0001-5202-7633")),
  person(given = "Rahul", family = "Satija", email = "rsatija@nygenome.org", role = "ctb", comment = c(ORCID = "0000-0001-9448-8833"))
  )
Description: A normalization method for single-cell UMI count data using a 
  variance stabilizing transformation. The transformation is based on a 
  negative binomial regression model with regularized parameters. As part of the
  same regression framework, this package also provides functions for
  batch correction, and data correction. See Hafemeister and Satija (2019)
  <doi:10.1186/s13059-019-1874-1>, and Choudhary and Satija (2022) <doi:10.1186/s13059-021-02584-9>
  for more details.
URL: https://github.com/satijalab/sctransform
BugReports: https://github.com/satijalab/sctransform/issues
License: GPL-3 | file LICENSE
Encoding: UTF-8
LazyData: true
Depends: R (>= 3.6.0)
LinkingTo: RcppArmadillo, Rcpp (>= 0.11.0)
SystemRequirements: C++17
Imports: dplyr, magrittr, MASS, Matrix (>= 1.5-0), methods,
        future.apply, future, parallelly, ggplot2, reshape2, rlang,
        gridExtra, matrixStats
Suggests: irlba, testthat, knitr
Enhances: glmGamPoi
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2025-12-25 12:01:11 UTC; saket
Author: Christoph Hafemeister [aut] (ORCID:
    <https://orcid.org/0000-0001-6365-8254>),
  Saket Choudhary [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-5202-7633>),
  Rahul Satija [ctb] (ORCID: <https://orcid.org/0000-0001-9448-8833>)
Maintainer: Saket Choudhary <saketc@iitb.ac.in>
Repository: CRAN
Date/Publication: 2026-01-10 07:50:30 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 11:25:04 UTC; unix
Archs: sctransform.so.dSYM
