# Generated by roxygen2: do not edit by hand

export(correct)
export(correct_counts)
export(diff_mean_test)
export(diff_mean_test_conserved)
export(generate)
export(get_model_var)
export(get_residual_var)
export(get_residuals)
export(plot_model)
export(plot_model_pars)
export(smooth_via_pca)
export(umify)
export(vst)
import(Matrix)
import(ggplot2)
import(reshape2)
importFrom(MASS,glm.nb)
importFrom(MASS,negative.binomial)
importFrom(MASS,theta.ml)
importFrom(MASS,theta.mm)
importFrom(dplyr,arrange)
importFrom(dplyr,case_when)
importFrom(dplyr,group_by)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,pull)
importFrom(dplyr,summarise)
importFrom(future,nbrOfWorkers)
importFrom(future.apply,future_lapply)
importFrom(graphics,abline)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(gridExtra,grid.arrange)
importFrom(magrittr,"%>%")
importFrom(matrixStats,rowMeans2)
importFrom(matrixStats,rowSds)
importFrom(matrixStats,rowVars)
importFrom(methods,as)
importFrom(rlang,.data)
importFrom(stats,aggregate)
importFrom(stats,anova)
importFrom(stats,approx)
importFrom(stats,approxfun)
importFrom(stats,as.formula)
importFrom(stats,bw.SJ)
importFrom(stats,bw.nrd0)
importFrom(stats,density)
importFrom(stats,df.residual)
importFrom(stats,glm)
importFrom(stats,glm.fit)
importFrom(stats,ksmooth)
importFrom(stats,mad)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,offset)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,poisson)
importFrom(stats,predict)
importFrom(stats,t.test)
importFrom(stats,var)
importFrom(utils,capture.output)
importFrom(utils,packageVersion)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
useDynLib(sctransform)
