# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

row_mean_dgcmatrix <- function(matrix) {
    .Call('_sctransform_row_mean_dgcmatrix', PACKAGE = 'sctransform', matrix)
}

row_mean_grouped_dgcmatrix <- function(matrix, group, shuffle) {
    .Call('_sctransform_row_mean_grouped_dgcmatrix', PACKAGE = 'sctransform', matrix, group, shuffle)
}

row_gmean_dgcmatrix <- function(matrix, eps) {
    .Call('_sctransform_row_gmean_dgcmatrix', PACKAGE = 'sctransform', matrix, eps)
}

row_gmean_grouped_dgcmatrix <- function(matrix, group, eps, shuffle) {
    .Call('_sctransform_row_gmean_grouped_dgcmatrix', PACKAGE = 'sctransform', matrix, group, eps, shuffle)
}

row_nonzero_count_dgcmatrix <- function(matrix) {
    .Call('_sctransform_row_nonzero_count_dgcmatrix', PACKAGE = 'sctransform', matrix)
}

row_nonzero_count_grouped_dgcmatrix <- function(matrix, group) {
    .Call('_sctransform_row_nonzero_count_grouped_dgcmatrix', PACKAGE = 'sctransform', matrix, group)
}

row_var_dgcmatrix <- function(x, i, rows, cols) {
    .Call('_sctransform_row_var_dgcmatrix', PACKAGE = 'sctransform', x, i, rows, cols)
}

grouped_mean_diff_per_row <- function(x, group, shuffle) {
    .Call('_sctransform_grouped_mean_diff_per_row', PACKAGE = 'sctransform', x, group, shuffle)
}

mean_boot <- function(x, N, S) {
    .Call('_sctransform_mean_boot', PACKAGE = 'sctransform', x, N, S)
}

mean_boot_grouped <- function(x, group, N, S) {
    .Call('_sctransform_mean_boot_grouped', PACKAGE = 'sctransform', x, group, N, S)
}

distribution_shift <- function(x) {
    .Call('_sctransform_distribution_shift', PACKAGE = 'sctransform', x)
}

qpois_reg <- function(X, Y, tol, maxiters, minphi, returnfit) {
    .Call('_sctransform_qpois_reg', PACKAGE = 'sctransform', X, Y, tol, maxiters, minphi, returnfit)
}

