% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{is_outlier}
\alias{is_outlier}
\title{Identify outliers}
\usage{
is_outlier(y, x, th = 10)
}
\arguments{
\item{y}{Dependent variable}

\item{x}{Independent variable}

\item{th}{Outlier score threshold}
}
\value{
Boolean vector
}
\description{
Identify outliers
}
