% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_model_formula}
\alias{get_model_formula}
\title{Extract model formula from model string}
\usage{
get_model_formula(model_str)
}
\arguments{
\item{model_str}{Model string with 'y' prefix (e.g., 'y ~ log_umi')}
}
\value{
Formula object without 'y' prefix
}
\description{
Helper function to convert model string (with 'y' prefix) to formula object
}
