% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{robust_scale}
\alias{robust_scale}
\title{Robust scale using median and mad}
\usage{
robust_scale(x)
}
\arguments{
\item{x}{Numeric}
}
\value{
Numeric
}
\description{
Robust scale using median and mad
}
