% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{row_var}
\alias{row_var}
\title{Variance per row}
\usage{
row_var(x)
}
\arguments{
\item{x}{matrix of class \code{matrix} or \code{dgCMatrix}}
}
\value{
variances
}
\description{
Variance per row
}
